/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.visual.widget;

import javax.annotation.Nonnull;
import java.awt.Point;
import org.netbeans.api.visual.anchor.Anchor;
import org.netbeans.api.visual.anchor.AnchorFactory;
import org.netbeans.api.visual.widget.ConnectionWidget;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import it.tidalwave.util.logging.Logger;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public abstract class ConnectingWidget extends ConnectionWidget
  {
    private static final String CLASS = ConnectingWidget.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private Point sourcePointCache;

    private Point targetPointCache;

    @Nonnull
    private final Widget sourceWidget;

    @Nonnull
    private final Widget targetWidget;

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ConnectingWidget (final @Nonnull Scene scene,
                             final @Nonnull Widget sourceWidget,
                             final @Nonnull Widget targetWidget)
      {
        super(scene);
        this.sourceWidget = sourceWidget;
        this.targetWidget = targetWidget;
        // lazy anchor location computation
        setSourceAnchor(AnchorFactory.createFixedAnchor(new Point(0, 0)));
        setTargetAnchor(AnchorFactory.createFixedAnchor(new Point(0, 0)));
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void fixAnchors()
      {
        logger.fine("fixAnchors()");

        final Point sourcePoint = createSourceAnchor().getRelatedSceneLocation();
        final Point targetPoint = createTargetAnchor().getRelatedSceneLocation();

        logger.finer(">>>> sourcePoint: %s vs %s", sourcePoint, sourcePointCache);
        logger.finer(">>>> targetPoint: %s vs %s", targetPoint, targetPointCache);

        if (!targetPoint.equals(targetPointCache))
          {
            setTargetAnchor(AnchorFactory.createFixedAnchor(targetPointCache = targetPoint));
          }

        if (!sourcePoint.equals(sourcePointCache))
          {
            setSourceAnchor(AnchorFactory.createFixedAnchor(sourcePointCache = sourcePoint));
          }

        logger.finer(">>>> fixAnchors() done");
      }

    @Nonnull
    protected Anchor createTargetAnchor()
      {
        return AnchorFactory.createCenterAnchor(targetWidget);
      }

    @Nonnull
    protected Anchor createSourceAnchor()
      {
        return AnchorFactory.createRectangularAnchor(sourceWidget);
      }
  }
